@@ -36,9 +36,12 @@
 #define __NR_SYSCALL_BASE	__NR_OABI_SYSCALL_BASE
 #endif
 
-/* NOTE: based on Linux 3.8.0-rc5 */
+/* NOTE: based on Linux 3.19 */
 const struct arch_syscall_def arm_syscall_table[] = { \
 	/* NOTE: arm_sync_file_range() and sync_file_range2() share values */
+	{ "_llseek", (__NR_SYSCALL_BASE + 140) },
+	{ "_newselect", (__NR_SYSCALL_BASE + 142) },
+	{ "_sysctl", (__NR_SYSCALL_BASE + 149) },
 	{ "accept", (__NR_SYSCALL_BASE + 285) },
 	{ "accept4", (__NR_SYSCALL_BASE + 366) },
 	{ "access", (__NR_SYSCALL_BASE + 33) },
@@ -52,8 +55,11 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "arch_prctl", __PNR_arch_prctl },
 	{ "bdflush", (__NR_SYSCALL_BASE + 134) },
 	{ "bind", (__NR_SYSCALL_BASE + 282) },
+	{ "bpf", (__NR_SYSCALL_BASE + 386) },
 	{ "break", __PNR_break },
 	{ "brk", (__NR_SYSCALL_BASE + 45) },
+	{ "cachectl", __PNR_cachectl },
+	{ "cacheflush", __PNR_cacheflush },
 	{ "capget", (__NR_SYSCALL_BASE + 184) },
 	{ "capset", (__NR_SYSCALL_BASE + 185) },
 	{ "chdir", (__NR_SYSCALL_BASE + 12) },
@@ -85,6 +91,7 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "eventfd", (__NR_SYSCALL_BASE + 351) },
 	{ "eventfd2", (__NR_SYSCALL_BASE + 356) },
 	{ "execve", (__NR_SYSCALL_BASE + 11) },
+	{ "execveat", (__NR_SYSCALL_BASE + 387) },
 	{ "exit", (__NR_SYSCALL_BASE +  1) },
 	{ "exit_group", (__NR_SYSCALL_BASE + 248) },
 	{ "faccessat", (__NR_SYSCALL_BASE + 334) },
@@ -144,6 +151,7 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "getpmsg", __PNR_getpmsg },
 	{ "getppid", (__NR_SYSCALL_BASE + 64) },
 	{ "getpriority", (__NR_SYSCALL_BASE + 96) },
+	{ "getrandom", (__NR_SYSCALL_BASE + 384) },
 	{ "getresgid", (__NR_SYSCALL_BASE + 171) },
 	{ "getresgid32", (__NR_SYSCALL_BASE + 211) },
 	{ "getresuid", (__NR_SYSCALL_BASE + 165) },
@@ -176,7 +184,8 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "ioprio_get", (__NR_SYSCALL_BASE + 315) },
 	{ "ioprio_set", (__NR_SYSCALL_BASE + 314) },
 	{ "ipc", (__NR_SYSCALL_BASE + 117) },
-	{ "kcmp", __PNR_kcmp },
+	{ "kcmp", (__NR_SYSCALL_BASE + 378) },
+	{ "kexec_file_load", __PNR_kexec_file_load },
 	{ "kexec_load", (__NR_SYSCALL_BASE + 347) },
 	{ "keyctl", (__NR_SYSCALL_BASE + 311) },
 	{ "kill", (__NR_SYSCALL_BASE + 37) },
@@ -188,7 +197,6 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "listen", (__NR_SYSCALL_BASE + 284) },
 	{ "listxattr", (__NR_SYSCALL_BASE + 232) },
 	{ "llistxattr", (__NR_SYSCALL_BASE + 233) },
-	{ "_llseek", (__NR_SYSCALL_BASE + 140) },
 	{ "lock", __PNR_lock },
 	{ "lookup_dcookie", (__NR_SYSCALL_BASE + 249) },
 	{ "lremovexattr", (__NR_SYSCALL_BASE + 236) },
@@ -198,6 +206,7 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "lstat64", (__NR_SYSCALL_BASE + 196) },
 	{ "madvise", (__NR_SYSCALL_BASE + 220) },
 	{ "mbind", (__NR_SYSCALL_BASE + 319) },
+	{ "memfd_create", (__NR_SYSCALL_BASE + 385) },
 	{ "migrate_pages", __PNR_migrate_pages },
 	{ "mincore", (__NR_SYSCALL_BASE + 219) },
 	{ "mkdir", (__NR_SYSCALL_BASE + 39) },
@@ -230,7 +239,6 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "munmap", (__NR_SYSCALL_BASE + 91) },
 	{ "name_to_handle_at", (__NR_SYSCALL_BASE + 370) },
 	{ "nanosleep", (__NR_SYSCALL_BASE + 162) },
-	{ "_newselect", (__NR_SYSCALL_BASE + 142) },
 	{ "newfstatat", __PNR_newfstatat },
 	{ "nfsservctl", (__NR_SYSCALL_BASE + 169) },
 	{ "nice", (__NR_SYSCALL_BASE + 34) },
@@ -239,6 +247,7 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "oldolduname", __PNR_oldolduname },
 	{ "oldstat", __PNR_oldstat },
 	{ "olduname", __PNR_olduname },
+	{ "oldwait4", __PNR_oldwait4 },
 	{ "open", (__NR_SYSCALL_BASE +  5) },
 	{ "open_by_handle_at", (__NR_SYSCALL_BASE + 371) },
 	{ "openat", (__NR_SYSCALL_BASE + 322) },
@@ -283,6 +292,7 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "removexattr", (__NR_SYSCALL_BASE + 235) },
 	{ "rename", (__NR_SYSCALL_BASE + 38) },
 	{ "renameat", (__NR_SYSCALL_BASE + 329) },
+	{ "renameat2", (__NR_SYSCALL_BASE + 382) },
 	{ "request_key", (__NR_SYSCALL_BASE + 310) },
 	{ "restart_syscall", (__NR_SYSCALL_BASE +  0) },
 	{ "rmdir", (__NR_SYSCALL_BASE + 40) },
@@ -297,13 +307,16 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "sched_get_priority_max", (__NR_SYSCALL_BASE + 159) },
 	{ "sched_get_priority_min", (__NR_SYSCALL_BASE + 160) },
 	{ "sched_getaffinity", (__NR_SYSCALL_BASE + 242) },
+	{ "sched_getattr", (__NR_SYSCALL_BASE + 381) },
 	{ "sched_getparam", (__NR_SYSCALL_BASE + 155) },
 	{ "sched_getscheduler", (__NR_SYSCALL_BASE + 157) },
 	{ "sched_rr_get_interval", (__NR_SYSCALL_BASE + 161) },
 	{ "sched_setaffinity", (__NR_SYSCALL_BASE + 241) },
+	{ "sched_setattr", (__NR_SYSCALL_BASE + 380) },
 	{ "sched_setparam", (__NR_SYSCALL_BASE + 154) },
 	{ "sched_setscheduler", (__NR_SYSCALL_BASE + 156) },
 	{ "sched_yield", (__NR_SYSCALL_BASE + 158) },
+	{ "seccomp", (__NR_SYSCALL_BASE + 383) },
 	{ "security", __PNR_security },
 	{ "select", (__NR_SYSCALL_BASE + 82) },
 	{ "semctl", (__NR_SYSCALL_BASE + 300) },
@@ -384,10 +397,10 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "sync_file_range2", (__NR_SYSCALL_BASE + 341) },
 	{ "syncfs", (__NR_SYSCALL_BASE + 373) },
 	{ "syscall", (__NR_SYSCALL_BASE + 113) },
-	{ "_sysctl", (__NR_SYSCALL_BASE + 149) },
 	{ "sysfs", (__NR_SYSCALL_BASE + 135) },
 	{ "sysinfo", (__NR_SYSCALL_BASE + 116) },
 	{ "syslog", (__NR_SYSCALL_BASE + 103) },
+	{ "sysmips", __PNR_sysmips },
 	{ "tee", (__NR_SYSCALL_BASE + 342) },
 	{ "tgkill", (__NR_SYSCALL_BASE + 268) },
 	{ "time", (__NR_SYSCALL_BASE + 13) },
@@ -396,6 +409,7 @@ const struct arch_syscall_def arm_syscall_table[] = { \
 	{ "timer_getoverrun", (__NR_SYSCALL_BASE + 260) },
 	{ "timer_gettime", (__NR_SYSCALL_BASE + 259) },
 	{ "timer_settime", (__NR_SYSCALL_BASE + 258) },
+	{ "timerfd", __PNR_timerfd },
 	{ "timerfd_create", (__NR_SYSCALL_BASE + 350) },
 	{ "timerfd_gettime", (__NR_SYSCALL_BASE + 354) },
 	{ "timerfd_settime", (__NR_SYSCALL_BASE + 353) },
@@ -477,3 +491,17 @@ const char *arm_syscall_resolve_num(int num)
 
 	return NULL;
 }
+
+/**
+ * Iterate through the syscall table and return the syscall name
+ * @param spot the offset into the syscall table
+ *
+ * Return the syscall name at position @spot or NULL on failure.  This function
+ * should only ever be used internally by libseccomp.
+ *
+ */
+const char *arm_syscall_iterate_name(unsigned int spot)
+{
+	/* XXX - no safety checks here */
+	return arm_syscall_table[spot].name;
+}
