@@ -26,6 +26,11 @@
 #include <string.h>
 #include <stdbool.h>
 
+#ifndef _BSD_SOURCE
+#define _BSD_SOURCE
+#endif
+#include <endian.h>
+
 #include <seccomp.h>
 
 #include "arch.h"
@@ -65,6 +70,8 @@ struct bpf_jump {
 	} tgt;
 	enum bpf_jump_type type;
 };
+#define _BPF_OP(a,x) \
+	(_htot16(a,x))
 #define _BPF_JMP_NO \
 	((struct bpf_jump) { .type = TGT_NONE })
 #define _BPF_JMP_NXT(x) \
@@ -77,8 +84,8 @@ struct bpf_jump {
 	((struct bpf_jump) { .type = TGT_PTR_BLK, .tgt = { .blk = (x) } })
 #define _BPF_JMP_HSH(x) \
 	((struct bpf_jump) { .type = TGT_PTR_HSH, .tgt = { .hash = (x) } })
-#define _BPF_K(x) \
-	((struct bpf_jump) { .type = TGT_K, .tgt = { .imm_k = (x) } })
+#define _BPF_K(a,x) \
+	((struct bpf_jump) { .type = TGT_K, .tgt = { .imm_k = _htot32(a,x) } })
 #define _BPF_JMP_MAX			255
 #define _BPF_JMP_MAX_RET		255
 
@@ -89,7 +96,7 @@ struct bpf_instr {
 	struct bpf_jump k;
 };
 #define _BPF_OFFSET_SYSCALL		(offsetof(struct seccomp_data, nr))
-#define _BPF_SYSCALL			_BPF_K(_BPF_OFFSET_SYSCALL)
+#define _BPF_SYSCALL(a)			_BPF_K(a,_BPF_OFFSET_SYSCALL)
 
 struct bpf_blk {
 	struct bpf_instr *blks;
@@ -171,6 +178,38 @@ static struct bpf_blk *_gen_bpf_chain(struct bpf_state *state,
 static struct bpf_blk *_hsh_remove(struct bpf_state *state, uint64_t h_val);
 static struct bpf_blk *_hsh_find(const struct bpf_state *state, uint64_t h_val);
 
+/**
+ * Convert a 16-bit host integer into the target's endianess
+ * @param arch the architecture definition
+ * @param val the 16-bit integer
+ *
+ * Convert the endianess of the supplied value and return it to the caller.
+ *
+ */
+uint16_t _htot16(const struct arch_def *arch, uint16_t val)
+{
+	if (arch->endian == ARCH_ENDIAN_LITTLE)
+		return htole16(val);
+	else
+		return htobe16(val);
+}
+
+/**
+ * Convert a 32-bit host integer into the target's endianess
+ * @param arch the architecture definition
+ * @param val the 32-bit integer
+ *
+ * Convert the endianess of the supplied value and return it to the caller.
+ *
+ */
+uint32_t _htot32(const struct arch_def *arch, uint32_t val)
+{
+	if (arch->endian == ARCH_ENDIAN_LITTLE)
+		return htole32(val);
+	else
+		return htobe32(val);
+}
+
 /**
  * Free the BPF instruction block
  * @param state the BPF state
@@ -451,6 +490,7 @@ static int _hsh_add(struct bpf_state *state, struct bpf_blk **blk_p,
 	h_new->found = (found ? 1 : 0);
 
 	/* insert the block into the hash table */
+hsh_add_restart:
 	h_iter = state->htbl[h_val & _BPF_HASH_MASK];
 	if (h_iter != NULL) {
 		do {
@@ -491,13 +531,14 @@ static int _hsh_add(struct bpf_state *state, struct bpf_blk **blk_p,
 					/* overflow */
 					blk->flag_hash = false;
 					blk->hash = 0;
+					free(h_new);
 					return -EFAULT;
 				}
 				h_val += ((uint64_t)1 << 32);
 				h_new->blk->hash = h_val;
 
 				/* restart at the beginning of the bucket */
-				h_iter = state->htbl[h_val & _BPF_HASH_MASK];
+				goto hsh_add_restart;
 			} else {
 				/* no match, move along */
 				h_prev = h_iter;
@@ -627,7 +668,8 @@ static struct bpf_blk *_gen_bpf_action(struct bpf_state *state,
 {
 	struct bpf_instr instr;
 
-	_BPF_INSTR(instr, BPF_RET, _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(action));
+	_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_RET),
+		   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(state->arch, action));
 	return _blk_append(state, blk, &instr);
 }
 
@@ -701,8 +743,9 @@ static struct bpf_blk *_gen_bpf_node(struct bpf_state *state,
 		/* reload the accumulator */
 		a_state->offset = acc_offset;
 		a_state->mask = ARG_MASK_MAX;
-		_BPF_INSTR(instr, BPF_LD + BPF_ABS,
-			   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(acc_offset));
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_LD + BPF_ABS),
+			   _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_K(state->arch, acc_offset));
 		blk = _blk_append(state, blk, &instr);
 		if (blk == NULL)
 			goto node_failure;
@@ -710,8 +753,9 @@ static struct bpf_blk *_gen_bpf_node(struct bpf_state *state,
 	if (acc_mask != a_state->mask) {
 		/* apply the bitmask */
 		a_state->mask = acc_mask;
-		_BPF_INSTR(instr, BPF_ALU + BPF_AND,
-			   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(acc_mask));
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_ALU + BPF_AND),
+			   _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_K(state->arch, acc_mask));
 		blk = _blk_append(state, blk, &instr);
 		if (blk == NULL)
 			goto node_failure;
@@ -721,16 +765,19 @@ static struct bpf_blk *_gen_bpf_node(struct bpf_state *state,
 	switch (node->op) {
 	case SCMP_CMP_MASKED_EQ:
 	case SCMP_CMP_EQ:
-		_BPF_INSTR(instr, BPF_JMP + BPF_JEQ,
-			   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(node->datum));
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_JMP + BPF_JEQ),
+			   _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_K(state->arch, node->datum));
 		break;
 	case SCMP_CMP_GT:
-		_BPF_INSTR(instr, BPF_JMP + BPF_JGT,
-			   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(node->datum));
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_JMP + BPF_JGT),
+			   _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_K(state->arch, node->datum));
 		break;
 	case SCMP_CMP_GE:
-		_BPF_INSTR(instr, BPF_JMP + BPF_JGE,
-			   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_K(node->datum));
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_JMP + BPF_JGE),
+			   _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_K(state->arch, node->datum));
 		break;
 	case SCMP_CMP_NE:
 	case SCMP_CMP_LT:
@@ -1021,8 +1068,9 @@ static struct bpf_blk *_gen_bpf_syscall(struct bpf_state *state,
 
 	/* setup the accumulator state */
 	if (acc_reset) {
-		_BPF_INSTR(instr, BPF_LD + BPF_ABS, _BPF_JMP_NO, _BPF_JMP_NO,
-			   _BPF_SYSCALL);
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_LD + BPF_ABS),
+			   _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_SYSCALL(state->arch));
 		blk_s = _blk_append(state, NULL, &instr);
 		if (blk_s == NULL)
 			return NULL;
@@ -1036,13 +1084,15 @@ static struct bpf_blk *_gen_bpf_syscall(struct bpf_state *state,
 
 	/* generate the argument chains */
 	blk_c = _gen_bpf_chain(state, sys, sys->chains, &def_jump, &a_state);
-	if (blk_c == NULL)
+	if (blk_c == NULL) {
+		_blk_free(state, blk_s);
 		return NULL;
+	}
 
 	/* syscall check */
-	_BPF_INSTR(instr, BPF_JMP + BPF_JEQ,
+	_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_JMP + BPF_JEQ),
 		   _BPF_JMP_HSH(blk_c->hash), _BPF_JMP_HSH(nxt_hash),
-		   _BPF_K(sys->num));
+		   _BPF_K(state->arch, sys->num));
 	blk_s = _blk_append(state, blk_s, &instr);
 	if (blk_s == NULL)
 		return NULL;
@@ -1149,8 +1199,8 @@ static struct bpf_blk *_gen_bpf_arch(struct bpf_state *state,
 		}
 	}
 
-	if ((db->arch->token == SCMP_ARCH_X86_64 ||
-	     db->arch->token == SCMP_ARCH_X32) && (db_secondary == NULL))
+	if ((state->arch->token == SCMP_ARCH_X86_64 ||
+	     state->arch->token == SCMP_ARCH_X32) && (db_secondary == NULL))
 		acc_reset = false;
 	else
 		acc_reset = true;
@@ -1186,27 +1236,29 @@ static struct bpf_blk *_gen_bpf_arch(struct bpf_state *state,
 	}
 
 	/* additional ABI filtering */
-	if ((db->arch->token == SCMP_ARCH_X86_64 ||
-	     db->arch->token == SCMP_ARCH_X32) && (db_secondary == NULL)) {
-		_BPF_INSTR(instr, BPF_LD + BPF_ABS, _BPF_JMP_NO, _BPF_JMP_NO,
-			   _BPF_SYSCALL);
+	if ((state->arch->token == SCMP_ARCH_X86_64 ||
+	     state->arch->token == SCMP_ARCH_X32) && (db_secondary == NULL)) {
+		_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_LD + BPF_ABS),
+			   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_SYSCALL(state->arch));
 		b_new = _blk_append(state, NULL, &instr);
 		if (b_new == NULL)
 			goto arch_failure;
-		if (db->arch->token == SCMP_ARCH_X86_64) {
+		if (state->arch->token == SCMP_ARCH_X86_64) {
 			/* filter out x32 */
-			_BPF_INSTR(instr, BPF_JMP + BPF_JGE,
+			_BPF_INSTR(instr,
+				   _BPF_OP(state->arch, BPF_JMP + BPF_JGE),
 				   _BPF_JMP_NXT(blk_cnt++), _BPF_JMP_NO,
-				   _BPF_K(X32_SYSCALL_BIT));
+				   _BPF_K(state->arch, X32_SYSCALL_BIT));
 			if (b_head != NULL)
 				instr.jf = _BPF_JMP_HSH(b_head->hash);
 			else
 				instr.jf = _BPF_JMP_HSH(state->def_hsh);
-		} else if (db->arch->token == SCMP_ARCH_X32) {
+		} else if (state->arch->token == SCMP_ARCH_X32) {
 			/* filter out x86_64 */
-			_BPF_INSTR(instr, BPF_JMP + BPF_JGE,
+			_BPF_INSTR(instr,
+				   _BPF_OP(state->arch, BPF_JMP + BPF_JGE),
 				   _BPF_JMP_NO, _BPF_JMP_NXT(blk_cnt++),
-				   _BPF_K(X32_SYSCALL_BIT));
+				   _BPF_K(state->arch, X32_SYSCALL_BIT));
 			if (b_head != NULL)
 				instr.jt = _BPF_JMP_HSH(b_head->hash);
 			else
@@ -1227,9 +1279,9 @@ static struct bpf_blk *_gen_bpf_arch(struct bpf_state *state,
 	}
 
 	/* do the ABI/architecture check */
-	_BPF_INSTR(instr, BPF_JMP + BPF_JEQ,
+	_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_JMP + BPF_JEQ),
 		   _BPF_JMP_NO, _BPF_JMP_NXT(blk_cnt++),
-		   _BPF_K(db->arch->token_bpf));
+		   _BPF_K(state->arch, state->arch->token_bpf));
 	if (b_head != NULL)
 		instr.jt = _BPF_JMP_HSH(b_head->hash);
 	else
@@ -1311,10 +1363,10 @@ static int _gen_bpf_build_jmp_ret(struct bpf_state *state,
 		j_len += b_jmp->blk_cnt;
 		b_jmp = b_jmp->next;
 	}
-	if (j_len <= _BPF_JMP_MAX_RET && b_jmp == blk_ret)
-		return 0;
 	if (b_jmp == NULL)
 		return -EFAULT;
+	if (j_len <= _BPF_JMP_MAX_RET && b_jmp == blk_ret)
+		return 0;
 
 	/* we need a closer return instruction, see if one already exists */
 	j_len = blk->blk_cnt - (offset + 1);
@@ -1324,10 +1376,10 @@ static int _gen_bpf_build_jmp_ret(struct bpf_state *state,
 		j_len += b_jmp->blk_cnt;
 		b_jmp = b_jmp->next;
 	}
-	if (j_len <= _BPF_JMP_MAX_RET && b_jmp->hash == tgt_hash)
-		return 0;
 	if (b_jmp == NULL)
 		return -EFAULT;
+	if (j_len <= _BPF_JMP_MAX_RET && b_jmp->hash == tgt_hash)
+		return 0;
 
 	/* we need to insert a new return instruction - create one */
 	b_new = _gen_bpf_action(state, NULL, blk_ret->blks[0].k.tgt.imm_k);
@@ -1382,7 +1434,8 @@ static int _gen_bpf_build_jmp(struct bpf_state *state,
 	if (b_tgt == blk)
 		return -EFAULT;
 
-	if (b_tgt->blk_cnt == 1 && b_tgt->blks[0].op == BPF_RET) {
+	if (b_tgt->blk_cnt == 1 &&
+	    b_tgt->blks[0].op == _BPF_OP(state->arch, BPF_RET)) {
 		rc = _gen_bpf_build_jmp_ret(state, blk, offset, b_tgt);
 		if (rc == 1)
 			return 1;
@@ -1397,10 +1450,10 @@ static int _gen_bpf_build_jmp(struct bpf_state *state,
 		jmp_len += b_jmp->blk_cnt;
 		b_jmp = b_jmp->next;
 	}
-	if (jmp_len <= _BPF_JMP_MAX && b_jmp == b_tgt)
-		return 0;
 	if (b_jmp == NULL)
 		return -EFAULT;
+	if (jmp_len <= _BPF_JMP_MAX && b_jmp == b_tgt)
+		return 0;
 
 	/* we need a long jump, see if one already exists */
 	jmp_len = blk->blk_cnt - (offset + 1);
@@ -1410,13 +1463,13 @@ static int _gen_bpf_build_jmp(struct bpf_state *state,
 		jmp_len += b_jmp->blk_cnt;
 		b_jmp = b_jmp->next;
 	}
-	if (jmp_len <= _BPF_JMP_MAX && b_jmp->hash == tgt_hash)
-		return 0;
 	if (b_jmp == NULL)
 		return -EFAULT;
+	if (jmp_len <= _BPF_JMP_MAX && b_jmp->hash == tgt_hash)
+		return 0;
 
 	/* we need to insert a long jump - create one */
-	_BPF_INSTR(instr, BPF_JMP + BPF_JA,
+	_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_JMP + BPF_JA),
 		   _BPF_JMP_NO, _BPF_JMP_NO, _BPF_JMP_HSH(tgt_hash));
 	b_new = _blk_append(state, NULL, &instr);
 	if (b_new == NULL)
@@ -1461,10 +1514,16 @@ static int _gen_bpf_build_bpf(struct bpf_state *state,
 	struct bpf_blk *b_badarch, *b_default;
 	struct bpf_blk *b_head = NULL, *b_tail = NULL, *b_iter, *b_new, *b_jmp;
 	struct db_filter *db_secondary = NULL;
+	struct arch_def pseudo_arch;
 
 	if (col->filter_cnt == 0)
 		return -EINVAL;
 
+	/* create a fake architecture definition for use in the early stages */
+	memset(&pseudo_arch, 0, sizeof(pseudo_arch));
+	pseudo_arch.endian = col->endian;
+	state->arch = &pseudo_arch;
+
 	/* generate the badarch action */
 	b_badarch = _gen_bpf_action(state, NULL, state->attr->act_badarch);
 	if (b_badarch == NULL)
@@ -1483,8 +1542,9 @@ static int _gen_bpf_build_bpf(struct bpf_state *state,
 	state->def_hsh = b_default->hash;
 
 	/* load the architecture token/number */
-	_BPF_INSTR(instr, BPF_LD + BPF_ABS, _BPF_JMP_NO, _BPF_JMP_NO,
-		   _BPF_K(offsetof(struct seccomp_data, arch)));
+	_BPF_INSTR(instr, _BPF_OP(state->arch, BPF_LD + BPF_ABS),
+		   _BPF_JMP_NO, _BPF_JMP_NO,
+		   _BPF_K(state->arch, offsetof(struct seccomp_data, arch)));
 	b_head = _blk_append(state, NULL, &instr);
 	if (b_head == NULL)
 		return -ENOMEM;
@@ -1532,6 +1592,9 @@ static int _gen_bpf_build_bpf(struct bpf_state *state,
 	b_tail->next = b_badarch;
 	b_tail = b_badarch;
 
+	/* reset the state to the pseudo_arch for the final resolution */
+	state->arch = &pseudo_arch;
+
 	/* resolve any TGT_NXT jumps at the top level */
 	b_iter = b_head;
 	do {
@@ -1684,7 +1747,7 @@ static int _gen_bpf_build_bpf(struct bpf_state *state,
 				}
 				if (b_jmp == NULL)
 					goto build_bpf_free_blks;
-				i_iter->k = _BPF_K(jmp_len);
+				i_iter->k = _BPF_K(state->arch, jmp_len);
 			}
 		}
 
