@@ -57,6 +57,20 @@ static const char *_pfc_arch(const struct arch_def *arch)
 		return "x32";
 	case SCMP_ARCH_ARM:
 		return "arm";
+	case SCMP_ARCH_AARCH64:
+		return "aarch64";
+	case SCMP_ARCH_MIPS:
+		return "mips";
+	case SCMP_ARCH_MIPSEL:
+		return "mipsel";
+	case SCMP_ARCH_MIPS64:
+		return "mips64";
+	case SCMP_ARCH_MIPSEL64:
+		return "mipsel64";
+	case SCMP_ARCH_MIPS64N32:
+		return "mips64n32";
+	case SCMP_ARCH_MIPSEL64N32:
+		return "mipsel64n32";
 	default:
 		return "UNKNOWN";
 	}
